/*!
        @file    $Id:: action_F_Overlap_Nf2.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-12 14:43:23 #$

        @version $LastChangedRevision: 929 $
*/

#ifndef ACTION_F_OVERLAP_NF2_INCLUDED
#define ACTION_F_OVERLAP_NF2_INCLUDED

#include <valarray>
#include "action.h"

#include "fopr_Overlap.h"
#include "force_F_Overlap_Nf2.h"

#include "index_lex.h"
#include "sortField_AbsLow.h"
#include "eigensolver_IRLanczos.h"
#include "solver_CG.h"
#include "shiftsolver_CG.h"
#include "math_Sign_Zolotarev.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Action for overlap fermion: temporary implementation.

/*!
   This is Action class dedicated to overlap fermion, as a
   temporary implementation.
   It may be not well organized and to be improved.
                                 [28 Dec 2011 H.Matsufuru]
 */

class Action_F_Overlap_Nf2 : public Action
{
 private:
  Field_G       *m_U;
  Index_lex     *m_index;
  RandomNumbers *m_rand;

  double             m_M0, m_kappa;
  std::valarray<int> m_boundary;
  Fopr_Wilson        *m_fopr_w;

  Field_F m_psf;

  double       m_mq, m_x_min, m_x_max, m_Stop_cond_ms;
  int          m_Np, m_Niter_ms;
  Fopr_Overlap *m_fopr_ov;

  Field m_force;
  int   m_status_linkv;

  // Zolotarev coefficients
  std::valarray<double> m_cl;
  std::valarray<double> m_bl;
  std::valarray<double> m_sigma;

  //  Eigenmode determination
  SortField_AbsLow      *m_sort;
  Eigensolver_IRLanczos *m_eigen;
  int m_Nmm;
  int m_Nsbt;
  std::valarray<double> m_TDa;
  std::valarray<Field>  m_vk;

 public:

  Action_F_Overlap_Nf2()
    : Action()
  {
    set_parameters();
  }

  ~Action_F_Overlap_Nf2()
  {
    delete m_fopr_w;
    delete m_index;
    delete m_fopr_ov;
    delete m_sort;
    delete m_eigen;
    //    delete m_eigen;
  }

  void set_parameters(const Parameters& param);
  void set_parameters();

  void set_config(Field *U)
  {
    m_U = (Field_G *)U;
    m_fopr_w->set_config(U);
  }

  void set_eigenmodes();

  void notify_linkv()
  {
    m_status_linkv = 0;
  }

  double langevin(RandomNumbers *);

  double calcH();

  const Field force();
  Field force_core(Field_F&, Field_F&);
  Field force_Wilson_core(Field_F&, Field_F&);
};
#endif
