/*!
        @file    $Id:: action_F_Staggered_eo.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-12 14:43:23 #$

        @version $LastChangedRevision: 929 $
*/


#ifndef ACTION_F_STAGGERED_EO_INCLUDED
#define ACTION_F_STAGGERED_EO_INCLUDED

#include "action.h"

#include "fopr_Staggered_eo.h"

#include "projection.h"
#include "smear.h"
#include "forceSmear.h"
#include "solver.h"

#include "mat_SU_N.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Action for staggered fermion: temporary implementation.

/*!
   This is Action class dedicated to staggered fermion in even-odd
   site index, as a temporary implementation.
   It may be not well organized and to be improved.
                                 [28 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.         [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_Action_F_Staggered_eo : virtual public Parameters
{
 public:
  Parameters_Action_F_Staggered_eo();
};
//- end

class Action_F_Staggered_eo : public Action {
 private:
  Field_G *m_U;
  int     m_status_linkv;

  Fopr  *m_fopr;
  Force *m_fopr_force;
  Field m_psf;
  Field m_force;

  Index_eo m_index_eo;

  Projection *m_proj;
  Smear      *m_smear;
  ForceSmear *m_force_smear;
  Solver     *m_solver;

  int                  m_Nsmear;
  double               m_rho;
  std::valarray<Field> m_Usmear;

  RandomNumbers *m_rand;

 public:
  Action_F_Staggered_eo(Fopr *fopr, Force *fopr_force) :
    Action()
  {
    m_fopr       = fopr;
    m_fopr_force = fopr_force;

    string str_proj_type = "Stout_SU3";
    m_proj = Projection::New(str_proj_type);

    string str_smear_type = "APE";
    m_smear       = Smear::New(str_smear_type, m_proj);
    m_force_smear = ForceSmear::New(str_smear_type, m_proj);

    string str_solver_type = "BiCGStab_Cmplx";
    m_solver = Solver::New(str_solver_type, m_fopr);
  }

  ~Action_F_Staggered_eo()
  {
    delete m_proj;
    delete m_smear;
    delete m_force_smear;
    delete m_solver;
  }

  void set_parameters(const Parameters& params);
  void set_parameters(const Parameters_Fopr_Staggered_eo& params);
  void set_parameters(int Nsmear, double rho);

  void set_config(Field *U)
  {
    m_U = (Field_G *)U;
    m_fopr->set_config(U);
  }

  void notify_linkv()
  {
    m_status_linkv = 0;
  }

  double langevin(RandomNumbers *);

  double calcH();

  const Field force();
};
#endif
