/*!
        @file    $Id:: fopr_Domainwall.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-12 14:43:23 #$

        @version $LastChangedRevision: 929 $
*/


#ifndef FOPR_DOMAINWALL_INCLUDED
#define FOPR_DOMAINWALL_INCLUDED

#include "fopr_Wilson.h"
#include "index_lex.h"
#include "math_Sign_Zolotarev.h"

#include "bridgeIO.h"
using Bridge::vout;

//- parameters class
class Parameters_Fopr_Domainwall : virtual public Parameters
{
 public:
  Parameters_Fopr_Domainwall();
};
//- end

//! Domain-wall fermion operator.

/*!
    This class implements domain-wall fermion operator.
    At present, only the standard domain-wall fermion
    is ready.
                                     [24 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.             [14 Nov 2012 Y.Namekawa]
 */

class Fopr_Domainwall : public Fopr
{
 private:
  // parameters common to overlap fermion
  double             m_mq;            // quark mass
  double             m_M0;            // domain-wall height
  int                m_Ns;            // size of fifth-dimension
  std::valarray<int> m_boundary;
  //  std::valarray<double> m_omega; // optimal domain-wall parameter
  std::string m_mode;

  Fopr_Wilson *m_fopr_w;

 public:

  Fopr_Domainwall(Fopr_Wilson *fopr_w)
    : Fopr(), m_fopr_w(fopr_w) {}

  void set_parameters(const Parameters& params);
  void set_parameters(const double mq, const double M0, const int Ns,
                      const std::valarray<int> bc);

  void set_config(Field *U)
  {
    m_fopr_w->set_config(U);
  }

  void set_mode(std::string mode)
  {
    m_mode = mode;
  }

  std::string get_mode() const
  {
    return m_mode;
  }

  void mult(Field& v, const Field& f)
  {
    v = mult(f);
  }

  void mult_dag(Field& v, const Field& f)
  {
    v = mult_dag(f);
  }

  const Field mult(const Field& f)
  {
    if (m_mode == "D") {
      return D(f);
    } else if (m_mode == "DdagD") {
      return DdagD(f);
    } else if (m_mode == "Ddag") {
      return Ddag(f);
    } else if (m_mode == "H") {
      return H(f);
    } else {
      vout.crucial(m_vl, "fopr_Domainwall: mode undefined.\n");
      abort();
    }
  }

  const Field mult_dag(const Field& f)
  {
    if (m_mode == "D") {
      return Ddag(f);
    } else if (m_mode == "DdagD") {
      return DdagD(f);
    } else if (m_mode == "Ddag") {
      return D(f);
    } else if (m_mode == "H") {
      return H(f);
    } else {
      vout.crucial(m_vl, "fopr_Domainwall: mode undefined.\n");
      abort();
    }
  }

  const Field DdagD(const Field&);
  const Field D(const Field&);
  const Field Ddag(const Field&);
  const Field H(const Field&);
  const Field mult_gm5R(const Field&);

  //! chiral projection of 4D field.
  //!   ipm = 1: positive - (1+gm5)/2 multiplied
  //!        -1: negative - (1-gm5)/2
  const Field mult_chproj_4d(const Field&, int ipm);

  int field_nvol() { return CommonParameters::Nvol(); }
  int field_nin() { return 2 * CommonParameters::Nc() * CommonParameters::Nd(); }
  int field_nex() { return m_Ns; }
};
#endif
