/*!
        @file    $Id:: force_F_Domainwall.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-12 14:43:23 #$

        @version $LastChangedRevision: 929 $
*/

#ifndef FORCE_F_DOMAINWALL_INCLUDED
#define FORCE_F_DOMAINWALL_INCLUDED

#include "force_F_Wilson_Nf2.h"
#include "fopr_Domainwall.h"
#include "index_lex.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Force calculation for domain-wall fermions.

/*!
    At present, only the standard domain-wall setting is
    available.
                                [28 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.        [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_Force_F_Domainwall : virtual public Parameters
{
 public:
  Parameters_Force_F_Domainwall();
};
//- end

class Force_F_Domainwall : public Force
{
 private:
  Field_G   *m_U;
  Index_lex *m_index;

  // parameters common to overlap fermion
  double             m_mq; // quark mass
  double             m_M0; // domain-wall height
  int                m_Ns; // size of fifth-dimension
  std::valarray<int> m_boundary;

  Fopr_Wilson        *m_fopr_w;
  Fopr_Domainwall    *m_fopr_dw;
  Force_F_Wilson_Nf2 *m_force_w;

 public:

  Force_F_Domainwall()
    : Force()
  {
    m_fopr_w  = new Fopr_Wilson;
    m_fopr_dw = new Fopr_Domainwall(m_fopr_w);
    m_force_w = new Force_F_Wilson_Nf2;
    m_boundary.resize(CommonParameters::Ndim());
  }

  ~Force_F_Domainwall()
  {
    delete m_force_w;
    delete m_fopr_dw;
    delete m_fopr_w;
  }

  void set_parameters(const Parameters& params);
  void set_parameters(const double mq, const double M0, const int Ns,
                      const std::valarray<int> bc);

  void set_config(Field *U)
  {
    m_U = (Field_G *)U;
    m_fopr_w->set_config(U);
    m_fopr_dw->set_config(U);
    m_force_w->set_config(U);
  }

  Field force_core(const Field& eta);

  Field force_udiv(const Field& eta);
  Field force_udiv1(const Field& zeta, const Field& eta);
};
#endif
