/*!
        @file    $Id:: force_F_Overlap_Nf2.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-12 14:43:23 #$

        @version $LastChangedRevision: 929 $
*/


#ifndef FORCE_F_OVERLAP_NF2_INCLUDED
#define FORCE_F_OVERLAP_NF2_INCLUDED

#include "force_F_Wilson_Nf2.h"
#include "fopr_Overlap.h"

#include "index_lex.h"
#include "solver_CG.h"
#include "shiftsolver_CG.h"
#include "math_Sign_Zolotarev.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Force calculation for overlap ferimon.

/*!
    At present, recursive calculation of smeqared fermion
    force is not implemented.
                              [28 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.      [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_Force_F_Overlap_Nf2 : virtual public Parameters
{
 public:
  Parameters_Force_F_Overlap_Nf2();
};
//- end

class Force_F_Overlap_Nf2 : public Force
{
 private:
  Field_G *m_U;

  double m_mq, m_M0;
  int    m_Np;
  double m_x_min, m_x_max;

  int    m_Niter_ms;
  double m_Stop_cond_ms;

  std::valarray<int> m_boundary;

  double m_kappa;

  Fopr_Wilson        *m_fopr_w;
  Force_F_Wilson_Nf2 *m_force_w;

  // Zolotarev coefficients
  std::valarray<double> m_cl;
  std::valarray<double> m_bl;
  std::valarray<double> m_sigma;

 public:

  Force_F_Overlap_Nf2()
    : Force() {}

  ~Force_F_Overlap_Nf2()
  {
    delete m_fopr_w;
    delete m_force_w;
  }

  void set_parameters(const Parameters& params);

//  void set_parameters (const Parameters_Fopr_Overlap& params);
  void set_parameters(const double mq, const double M0, const int Np,
                      const double x_min, const double x_max,
                      const int Niter_ms, const double Stop_cond_ms,
                      const std::valarray<int> bc);
  void set_parameters();

  void set_config(Field *U)
  {
    m_U = (Field_G *)U;
    m_fopr_w->set_config(U);
    m_force_w->set_config(U);
  }

  //  void set_eigenmodes();

  Field force_core(const Field&);
  Field force_core1(const Field_F&, const Field_F&);

  Field force_udiv(const Field&) { return Field(); }  // dummy entry
};
#endif
