/*!
        @file    $Id:: force_F_TMWilson_Nf2.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-12 14:43:23 #$

        @version $LastChangedRevision: 929 $
*/

#ifndef FORCE_TMFWILSON_NF2_INCLUDED
#define FORCE_TMFWILSON_NF2_INCLUDED

#include "force.h"
#include "fopr_TMWilson.h"

#include "tensorProd.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Force for the twisted-mass Wilson fermion operator

/*!
    This class calculate the force of the twisted-mass Wilson
    fermion.
    The gamma matrix representation is given as control string
    "Dirac"(default) or "Chiral" at the construction, which is
    used to construct the Fopr_Wilson instance.
                                     [23 Dec 2011 H.Matusfuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.             [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_Force_F_TMWilson_Nf2 : virtual public Parameters
{
 public:
  Parameters_Force_F_TMWilson_Nf2();
};
//- end

class Force_F_TMWilson_Nf2 : public Force
{
 private:
  Field_G *m_U;

  double             m_kappa;
  double             m_tw_mass;
  std::valarray<int> m_boundary;
  Fopr_TMWilson      *m_fopr_tw;
  Field_F            m_psf;
  std::string        m_repr;

 public:

  Force_F_TMWilson_Nf2()
    : Force()
  {
    m_repr    = "Dirac";
    m_fopr_tw = new Fopr_TMWilson(m_repr);
    m_boundary.resize(CommonParameters::Ndim());
  }

  Force_F_TMWilson_Nf2(std::string repr)
    : Force()
  {
    m_repr    = repr;
    m_fopr_tw = new Fopr_TMWilson(m_repr);
    m_boundary.resize(CommonParameters::Ndim());
  }

  ~Force_F_TMWilson_Nf2()
  {
    delete m_fopr_tw;
  }

  void set_parameters(const Parameters& params);

  // void set_parameters (const Parameters_Fopr_TMWilson& params);
  void set_parameters(const double kappa, const double tw_mass,
                      const std::valarray<int> bc);

  void set_config(Field *U)
  {
    m_U = (Field_G *)U;
    m_fopr_tw->set_config(U);
  }

  Field force_core(const Field& eta);
  Field force_core1(const Field& zeta, const Field& eta);

  Field force_udiv(const Field& eta);
  Field force_udiv1(const Field& zeta, const Field& eta);

 private:
  Field force_udiv1(const Field_F& zeta, const Field_F& eta);
};
#endif
