/*!
        @file    $Id:: integrator_Omelyan.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-18 11:55:46 #$

        @version $LastChangedRevision: 934 $
*/

#ifndef INTEGRATOR_OMELYAN_INCLUDED
#define INTEGRATOR_OMELYAN_INCLUDED

#include "action.h"
#include "director.h"
#include "randomNumbers.h"
#include "integrator.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Omelyan integrator to compose MD integrator.

/*!
                             [28 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.     [03 Mar 2013 Y.Namekawa]
 */

//- parameters class
class Parameters_Integrator_Omelyan : virtual public Parameters
{
 public:
  Parameters_Integrator_Omelyan();
};
//- end

class Integrator_Omelyan : public Integrator
{
 private:
  double                  m_Estep;  // step size of each molecular dynamical evolution
  int                     m_Nstep;  // number of steps
  int                     m_level;  // level number
  double                  m_lambda; // Omelyan's optimized parameter
  Integrator              *m_integ_next;
  std::valarray<Action *> m_action; // actions

 public:
  //! constructor with actions and next level integrator.
  Integrator_Omelyan(std::valarray<Action *> action,
                     Integrator *integ_next)
  {
    m_action.resize(action.size());
    for (int i = 0; i < action.size(); ++i) {
      m_action[i] = action[i];
    }
    m_integ_next = integ_next;
    m_Estep      = 0.0;
    m_Nstep      = 0;
    m_lambda     = 0.0;
  }

  //! destructor
  ~Integrator_Omelyan() {}

  void set_parameters(const Parameters& params);
  void set_parameters(int level, double Estep, int Nstep,
                      double lambda_Omelyan);

  void evolve(Field_G& iP, Field_G& U);
};
#endif
