/*!
        @file    $Id:: corr2pt_Staggered.cpp #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-07-18 11:55:46 #$

        @version $LastChangedRevision: 934 $
*/

#include "corr2pt_Staggered.h"
#include "communicator.h"

using std::valarray;
using Bridge::vout;

//====================================================================
void Corr2pt_Staggered::meson(valarray<double>& meson,
                              const valarray<Field_F_1spinor>& sq1,
                              const valarray<Field_F_1spinor>& sq2)
{
  int Nc   = CommonParameters::Nc();
  int Nvol = CommonParameters::Nvol();
  int Lt   = CommonParameters::Lt();
  int Nx   = CommonParameters::Nx();
  int Ny   = CommonParameters::Ny();
  int Nz   = CommonParameters::Nz();
  int Nt   = CommonParameters::Nt();

  meson.resize(Lt);

  valarray<double> corr_local(Nt);
  Field            corrF(2, Nvol, 2);

  {
    int isrc = 0;
    //for(int isrc = 0; isrc < 2; ++ isrc){

    int ex = 0;

    for (int site = 0; site < Nvol; ++site) {
      double corr_r = 0.0;
      double corr_i = 0.0;
      for (int c0 = 0; c0 < Nc; ++c0) {
        for (int c1 = 0; c1 < Nc; ++c1) {
          corr_r += sq1[c0 + Nc * isrc].cmp_r(c1, site, ex)
                    * sq2[c0 + Nc * isrc].cmp_r(c1, site, ex)
                    + sq1[c0 + Nc * isrc].cmp_i(c1, site, ex)
                    * sq2[c0 + Nc * isrc].cmp_i(c1, site, ex);
          corr_i += sq1[c0 + Nc * isrc].cmp_r(c1, site, ex)
                    * sq2[c0 + Nc * isrc].cmp_i(c1, site, ex)
                    - sq1[c0 + Nc * isrc].cmp_i(c1, site, ex)
                    * sq2[c0 + Nc * isrc].cmp_r(c1, site, ex);
        }
      }
      corrF.set(0, site, ex, corr_r);
      corrF.set(1, site, ex, corr_i);
    }

    for (int t = 0; t < Nt; ++t) {
      corr_local[t] = 0.0;
      for (int z = 0; z < Nz; ++z) {
        for (int y = 0; y < Ny; ++y) {
          for (int x = 0; x < Nx; ++x) {
            int site = m_index.site(x, y, z, t);
            corr_local[t] += corrF.cmp(0, site, ex);
          }
        }
      }
    }

    valarray<double> corr_tmp(Lt);

    int ipet = Communicator::ipe(3);
    for (int t = 0; t < Lt; ++t) {
      corr_tmp[t] = 0.0;
    }
    for (int t = 0; t < Nt; ++t) {
      int t2 = t + ipet * Nt;
      corr_tmp[t2] = corr_local[t];
    }
    for (int t = 0; t < Lt; ++t) {
      double crr = corr_tmp[t];
      meson[t] = Communicator::reduce_sum(crr);
    }
  }
}


//====================================================================
//============================================================END=====
