/*!
        @file    $Id:: fprop_Overlap_5d.h #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-04-27 12:28:50 #$

        @version $LastChangedRevision: 875 $
*/

#ifndef FPROP_OVERLAP_5D_INCLUDED
#define FPROP_OVERLAP_5D_INCLUDED

#include "fopr_Overlap.h"
#include "fopr_Overlap_5d.h"

#include "index_lex.h"
#include "index_eo.h"

#include "solver.h"

#include "bridgeIO.h"
using Bridge::vout;

//! Calculation of overlap propagater with 5d solver.

/*!
   This class calculate overlap quark propagator using
   5-dimensional solver.
                                 [25 Dec 2011 H.Matsufuru]
    (Coding history will be recovered from trac.)
    YAML is implemented.         [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_Fprop_Overlap_5d : virtual public Parameters
{
 public:
  Parameters_Fprop_Overlap_5d();
};
//- end

class Fprop_Overlap_5d
{
 protected:
  Bridge::VerboseLevel m_vl;

 private:
  //- parameters common to overlap fermion
  double             m_mq;             // quark mass
  double             m_M0;             // domain-wall height
  int                m_Np;             // number of poles in rational approx.
  double             m_x_min, m_x_max; // valid range of approximate sign function
  int                m_Niter_ms;       // max iteration of shiftsolver (dummy)
  double             m_Stop_cond_ms;   // stopping condition of shift solver (dummy)
  std::valarray<int> m_boundary;

  //- solver parameter
  Parameters m_params_solver;

  Fopr_Overlap *m_fopr_ov;
  Field_G      *m_conf;

  Fopr_Overlap_5d *m_fopr_ov5d;
  Fopr_Wilson_eo  *m_fopr_w_eo;
  Field_G         *m_Ueo;

  Index_lex m_index;
  Index_eo  m_index_eo;

  //- low-mode subtraction
  int m_Nsbt;
  std::valarray<double> *m_ev;
  std::valarray<Field>  *m_vk;

 public:
  Fprop_Overlap_5d(Fopr_Overlap *foprov, Field_G *conf)
    : m_vl(CommonParameters::Vlevel()),
      m_fopr_ov(foprov),
      m_conf(conf) {}

  void set_parameters(const Parameters& params_fprop_overlap_5d);
  void set_parameters(const Parameters& params_overlap,
                      const Parameters& params_solver);
  void set_parameters(const double mq, const double M0, const int Np,
                      const double x_min, const double x_max,
                      const int Niter_ms, const double Stop_cond_ms,
                      const std::valarray<int> bc);

  void calc_fprop(Field&, const Field&);
  void calc_H2inv(Field&, const Field&);

  void solve_overlap_5D_1(Field&, const Field&, int&, double&, double&);

  void set_lowmodes(int Nsbt, std::valarray<double> *ev,
                    std::valarray<Field> *vk)
  {
    m_Nsbt = Nsbt;
    m_ev   = ev;
    m_vk   = vk;
  }

 private:
  void set_env();
};
#endif
