/*!
        @file    $Id:: fprop_Staggered_eo.cpp #$

        @brief

        @author  <Hideo Matsufuru> hideo.matsufuru@kek.jp(matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate:: 2013-04-27 12:28:50 #$

        @version $LastChangedRevision: 875 $
*/

#include "fprop_Staggered_eo.h"

//====================================================================
void Fprop_Staggered_eo::set_parameters(const Parameters& params_solver)
{
  m_params_solver = params_solver;
}


//====================================================================
void Fprop_Staggered_eo::calc(Field_F_1spinor& xq, const Field_F_1spinor& b)
{
  // ####  parameter setup  ####
  int Nvol = CommonParameters::Nvol();

  const string str_solver_type = m_params_solver.get_string("solver_type");

  // ####  object setup  ####
  Solver *solver = Solver::New(str_solver_type, m_fopr_eo);

  solver->set_parameters(m_params_solver);

  int    Nconv;
  double diff;

  Field_F_1spinor be(Nvol / 2, 1);
  Field_F_1spinor bo(Nvol / 2, 1);
  Field_F_1spinor xqe(Nvol / 2, 1);
  Field_F_1spinor xqo(Nvol / 2, 1);

  m_index_eo.convertField(be, b, 0);
  m_index_eo.convertField(bo, b, 1);

  vout.general(m_vl, "be norm = %18.8e\n", be.norm2());
  vout.general(m_vl, "bo norm = %18.8e\n", bo.norm2());

  be -= (Field_F_1spinor)m_fopr_eo->Meo(bo, 0);

  vout.general(m_vl, "be norm = %18.8e\n", be.norm2());

  m_fopr_eo->set_mode("Dee");
  solver->solve(xqe, be, Nconv, diff);

  xqo  = bo;
  xqo -= (Field_F_1spinor)m_fopr_eo->Meo(xqe, 1);
  m_index_eo.reverseField(xq, xqe, 0);
  m_index_eo.reverseField(xq, xqo, 1);
  //  m_fopr_eo->fprop_normalize(xq);

  vout.general(m_vl, "  Nconv = %d\n", Nconv);
  vout.general(m_vl, "  diff  = %.8e\n", diff);


  // ####  tidy up  ####
  delete solver;
}


//====================================================================
//============================================================END=====
